@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# $Id: owfs.py 1020 2014-08-26 18:20:40Z mwall $
+# $Id: owfs.py 1159 2014-12-04 04:40:32Z mwall $
 #
 # Copyright 2013 Matthew Wall
 # Thanks to Mark Cressey (onewireweewx) and Howard Walter (TAI code).
@@ -16,24 +16,23 @@
 
 """Classes and functions for interfacing with one-wire sensors via owfs.
 
-This driver will read raw data from owfs, then the weewx calibration can be
-used to convert the raw values to desired scaling or units.  Mapping from
-one-wire device and attribute to weewx database field is done in the OWFS
-section of weewx.conf.
+This file contains both a weewx driver and a weewx service.  Either will
+read raw data from owfs, then the weewx calibration can be used to adjust
+raw values as needed.  Mapping from one-wire device and attribute to weewx
+database field is done in the OWFS section of weewx.conf.
 
-This module requires one-wire file system including python bindings.  The
-owftpd, owhttpd, and owserver services are not used and do not need to be
-running.
+This module requires the python bindings for owfs.  The owfs software itself
+can be installed if you like, but it is not necessary; the owftpd, owhttpd,
+and owserver services are not used and do not need to be running.  In fact,
+it is safer to leave those services disabled to ensure that they do not
+conflict with weewx when it attempts to read one-wire devices.
 
 On debian systems, install the python bindings with something like this:
 
-sudo apt-get install owfs python-ow
+sudo apt-get install python-ow
 
-Put this file in the weewx 'user' directory, then modify weewx.conf with
-something like this:
-
-[Station]
-    station_type = OWFS
+Put this file, owfs.py, in the weewx 'user' directory, then modify weewx.conf
+with something like this:
 
 [OWFS]
     interface = u
@@ -41,6 +40,30 @@ something like this:
     [[sensor_map]]
         inTemp = /uncached/28.8A071E050000/temperature
 
+To use as a driver:
+
+[Station]
+    station_type = OWFS
+
+To use as a service:
+
+[Engine]
+    [[Service]]
+        data_services = user.owfs.OWFSService
+
+The service can be bound to LOOP or ARCHIVE.  The default binding is archive;
+when a new archive record is created, this service is invoked to collect data
+from the one-wire bus.  The other option is loop binding; when loop data are
+collected, this service is invoked to collect data from the one-wire bus.
+
+Beware that a slow one-wire bus can adversely affect the performance and
+behavior of weewx, especially when using loop binding.
+
+Use the binding parameter to indicate LOOP or ARCHIVE binding.  For example,
+
+[OWFS]
+    binding = loop
+
 Only numeric sensor values are supported.
 
 By default, data from each sensor are treated as gauge.  Data types include:
@@ -50,10 +73,21 @@ By default, data from each sensor are treated as gauge.  Data types include:
   average - time average by calculating the delta divided by time period
   counter - difference between current and last, always increasing
 
+By default, the units of the sensors is assumed to be metric.  If the sensors
+have been configured to output in US units, use the unit_system option.  For
+example,
+
+[OWFS]
+    unit_system = US
+
 The interface indicates where the one-wire devices are attached.  The default
 value is u, which is shorthand for 'usb'.  This is the option to use for a
 DS9490R USB adaptor.  Other options include a serial port such as /dev/ttyS0,
-or remote_system:3003 to get data from a remote host running owserver.
+or remote_system:3003 to get data from a remote host running owserver.  For
+example,
+
+[OWFS]
+    interface = /dev/ttyS0
 
 The sensor map is simply a list of database field followed by full path to the
 desired sensor reading.  Only sensor values that can be converted to float
@@ -83,76 +117,62 @@ Example Configurations
 AAG TAI8515 V3 weather station
 
 [OWFS]
-    driver = user.owfs
     [[sensor_map]]
         outTemp = /uncached/XX.YYYYYYYYYYYY/temperature
-        windDir = /uncached/XX.YYYYYYYYYYYY/volt.ALL
-        windSpeed = /uncached/XX.YYYYYYYYYYYY/counters.A
+        windDir = /uncached/XX.YYYYYYYYYYYY              # volt.ALL
+        windSpeed = /uncached/XX.YYYYYYYYYYYY            # counters.A
     [[sensor_type]]
         windDir = aag_windvane
-        windSpeed = average
-
-[StdCalibrate]
-    [[Corrections]]
-        windSpeed = windSpeed * 3.948 / 2 # speed in mph is 2.453 * cnt / 2
+        windSpeed = aag_windspeed
 
 Hobby-Boards ADS wind instrument, lightning sensor, solar radiation
 sensor, and rainwise rain instrument
 
 [OWFS]
-    driver = user.owfs
     [[sensor_map]]
         inTemp = /XX.YYYYYYYYYYYY/temperature
         UV = /XX.YYYYYYYYYYYY/UVI/UVI
         luminosity = /XX.YYYYYYYYYYYY/S3-R1-A/luminosity
         lightning = /XX.YYYYYYYYYYYY/counters.A
-        rain = /XX.YYYYYYYYYYYY/counters.B
-        windDir = /XX.YYYYYYYYYYYY
-        windSpeed = /XX.YYYYYYYYYYYY/counters.A
+        rain = /XX.YYYYYYYYYYYY                          # counters.B
+        windDir = /XX.YYYYYYYYYYYY                       # VAD
+        windSpeed = /XX.YYYYYYYYYYYY                     # counters.A
     [[sensor_type]]
         lightning = counter
-        rain = delta
+        rain = rainwise_bucket
         windDir = ads_windvane
-        windSpeed = average
+        windSpeed = ads_windspeed
 
 [StdCalibrate]
     [[Corrections]]
         radiation = luminosity * 1.730463
-        rain = rain * 0.0254 # rainwise bucket is 0.01 inches per tip
-        rainRate = rainRate * 0.0254
-        windSpeed = windSpeed * 2.01168 # convert from mph to kph
 
 Hobby-Boards with Inspeed wind instrument
 
 [OWFS]
-    driver = user.owfs
     [[sensor_map]]
-        windDir = /XX.YYYYYYYYYYYY
-        windSpeed = /XX.YYYYYYYYYYYY/counters.A
+        windDir = /XX.YYYYYYYYYYYY                       # VAD / VDD
+        windSpeed = /XX.YYYYYYYYYYYY                     # counters.A
     [[sensor_type]]
         windDir = inspeed_windvane
-        windSpeed = average
-
-[StdCalibrate]
-    [[Corrections]]
-        windSpeed = windSpeed * 4.02336 # convert from mph to kph
+        windSpeed = inspeed_windspeed
 """
 
-# FIXME: push the derived calculations into a generic weewx service
-# FIXME: provide other options for rainRate calculation (e.g. 5 minute window)
-# FIXME: automatically detect the sensor type (adverse performance?)
-# FIXME: some people do a owfs_put of 15 to branch.BYTE.  what does this do?
+# FIXME: automatically detect each sensor type
+# FIXME: automatically detect per-sensor units
 
 import syslog
 import time
 import ow
-import weewx.abstractstation
-import weewx.wxformulas
 
-DRIVER_VERSION = 'X'
+import weewx
+from weewx.drivers import AbstractDevice
+from weewx.engine import StdService
 
-def logmsg(dst, msg):
-    syslog.syslog(dst, 'owfs: %s' % msg)
+VERSION = "0.11"
+
+def logmsg(level, msg):
+    syslog.syslog(level, 'owfs: %s' % msg)
 
 def logdbg(msg):
     logmsg(syslog.LOG_DEBUG, msg)
@@ -160,9 +180,6 @@ def logdbg(msg):
 def loginf(msg):
     logmsg(syslog.LOG_INFO, msg)
 
-def logcrt(msg):
-    logmsg(syslog.LOG_CRIT, msg)
-
 def logerr(msg):
     logmsg(syslog.LOG_ERR, msg)
 
@@ -213,6 +230,18 @@ def average(key, path, last_data, ts):
     last_data[key] = (v, ts)
     return x
 
+def rainwise_bucket(key, path, last_data, ts):
+    cnt = delta(key, "%s%s" % (path, "/counters.B"), last_data, ts)
+    if cnt is not None:
+        cnt *= 0.0254 # rainwise bucket is 0.01 inches per tip, convert to cm
+    return cnt
+
+def ads_windspeed(key, path, last_data, ts):
+    ws = average(key, "%s%s" % (path, "/counters.A"), last_data, ts)
+    if ws is not None:
+        ws *= 2.01168 # convert to kph
+    return ws
+
 def ads_winddir(key, path, last_data, ts):
     """Calculate wind direction for Hobby-Boards ADS wind instrument.
     Get wind direction from the VAD register fo the DS2438.
@@ -253,6 +282,12 @@ def ads_winddir(key, path, last_data, ts):
         wdir = 337.5 # NNW
     return wdir
 
+def inspeed_windspeed(key, path, last_data, ts):
+    ws = average(key, "%s%s" % (path, "/counters.A"), last_data, ts)
+    if ws is not None:
+        ws *= 4.02336 # convert to kph
+    return ws
+
 def inspeed_winddir(key, path, last_data, ts):
     """Calculate wind direction for Hobby-Boards Inspeed wind instrument.
     Get wind direction from the VDD and VAD register of DS2438.
@@ -262,10 +297,16 @@ def inspeed_winddir(key, path, last_data, ts):
     d = (400*(vad-0.05*vdd))/vdd
     return d
 
+def aag_windspeed(key, path, last_data, ts):
+    ws = average(key, "%s%s" % (path, "/counters.A"), last_data, ts)
+    if ws is not None:
+        ws *= 3.948 / 2 # speed in mph is 2.453 * cnt / 2; convert to kph
+    return ws
+
 def aag_winddir(key, path, last_data, ts):
     """Calculate wind direction for AAG TAI8515 V3 wind instrument.
     Contributed by Howard Walter, based on oww C implementation."""
-    w = ow.owfs_get(path)
+    w = ow.owfs_get("%s%s" % (path, "/volt.ALL"))
     wd = w.split(',')
     wd = [float(x) for x in wd]
     mx = max(x for x in wd)
@@ -318,12 +359,24 @@ def aag_winddir(key, path, last_data, ts):
                                     wdir = 1
     return 22.5 * wdir
 
-def loader(config_dict, engine):
-    altitude_m = weewx.units.getAltitudeM(config_dict)
-    station = OWFS(altitude=altitude_m, **config_dict['OWFS'])
-    return station
+SENSOR_TYPES = {
+    'gauge': gauge,
+    'delta': delta,
+    'average': average,
+    'counter': counter,
+    'ads_windvane': ads_winddir,
+    'ads_windspeed': ads_windspeed,
+    'inspeed_windvane': inspeed_winddir,
+    'inspeed_windspeed': inspeed_windspeed,
+    'aag_windvane': aag_winddir,
+    'aag_windspeed': aag_windspeed,
+    'rainwise_bucket': rainwise_bucket,
+    }
 
-class OWFS(weewx.abstractstation.AbstractStation):
+def loader(config_dict, engine):
+    return OWFSDriver(**config_dict['OWFS'])
+
+class OWFSDriver(weewx.drivers.AbstractDevice):
     """Driver for one-wire sensors via owfs."""
     
     def __init__(self, **stn_dict) :
@@ -341,39 +394,34 @@ class OWFS(weewx.abstractstation.AbstractStation):
 
         polling_interval: How often to poll for data, in seconds.
         [Optional. Default is 10]
+
+        unit_system: The unit system the data are assumed to be in.  Can
+        be one of 'METRIC' or 'US'.  This assumes that all sensors are
+        reporting data in the same unit system.
+        [Optional. Default is METRIC]
         """
-        self.altitude          = stn_dict['altitude']
-        self.sensor_map        = stn_dict['sensor_map']
-        self.sensor_type       = stn_dict.get('sensor_type', {})
-        self.interface         = stn_dict.get('interface', 'u')
-        self.polling_interval  = int(stn_dict.get('polling_interval', 10))
-        self.max_tries         = int(stn_dict.get('max_tries', 3))
-        self.retry_wait        = int(stn_dict.get('retry_wait', 5))
-        self.pressure_offset   = stn_dict.get('pressure_offset', None)
-        if self.pressure_offset is not None:
-            self.pressure_offset = float(self.pressure_offset)
-        self.last_data         = {}
-        self.last_rain_ts      = None
+        self.sensor_map = stn_dict['sensor_map']
+        self.sensor_type = stn_dict.get('sensor_type', {})
+        self.interface = stn_dict.get('interface', 'u')
+        self.polling_interval = int(stn_dict.get('polling_interval', 10))
+        self.max_tries = int(stn_dict.get('max_tries', 3))
+        self.retry_wait = int(stn_dict.get('retry_wait', 5))
+        self.unit_system = stn_dict.get('unit_system', 'METRIC').lower()
+        self.last_data = {}
+        self.units = weewx.US if self.unit_system == 'us' else weewx.METRIC
 
-        self.sensor_types = {
-            'gauge' : gauge,
-            'delta' : delta,
-            'average' : average,
-            'counter' : counter,
-            'ads_windvane' : ads_winddir,
-            'inspeed_windvane' : inspeed_winddir,
-            'aag_windvane' : aag_winddir,
-            }
-
-        loginf('driver version is %s' % DRIVER_VERSION)
+        loginf('driver version is %s' % VERSION)
         loginf('interface is %s' % self.interface)
-        loginf('polling interval is %s' % str(self.polling_interval))
         loginf('sensor map is %s' % self.sensor_map)
         loginf('sensor type map is %s' % self.sensor_type)
-        loginf('altitude is %s meters' % str(self.altitude))
-        loginf('pressure offset is %s mbar' % str(self.pressure_offset))
+        loginf('polling interval is %s' % str(self.polling_interval))
+        loginf('sensor unit system is %s' % self.unit_system)
         ow.init(self.interface)
 
+        # open all 1-wire channels on a Hobby Boards 4-channel hub.  see:
+        #   http://owfs.org/index.php?page=4-channel-hub
+#        ow.owfs_put("%s/hub/branch.BYTE" % hubpath, 15)
+
     @property
     def hardware_name(self):
         return 'OWFS'
@@ -384,15 +432,14 @@ class OWFS(weewx.abstractstation.AbstractStation):
             ntries += 1
             try:
                 last_data = dict(self.last_data)
-                p = {}
-                p['usUnits'] = weewx.METRIC
-                p['dateTime'] = int(time.time() + 0.5)
+                p = {'usUnits': self.units,
+                     'dateTime': int(time.time() + 0.5)}
                 for s in self.sensor_map:
                     st = 'gauge'
                     if s in self.sensor_type:
                         st = self.sensor_type[s]
-                    if st in self.sensor_types:
-                        func = self.sensor_types[st]
+                    if st in SENSOR_TYPES:
+                        func = SENSOR_TYPES[st]
                         p[s] = func(s, self.sensor_map[s],
                                     last_data, p['dateTime'])
                     else:
@@ -413,35 +460,99 @@ class OWFS(weewx.abstractstation.AbstractStation):
             raise weewx.RetriesExceeded(msg)
 
     def calculate_derived(self, data):
-        if 'pressure' in data:
-            adjp = data['pressure']
-            if self.pressure_offset is not None and adjp is not None:
-                adjp += self.pressure_offset
-            data['altimeter'] = weewx.wxformulas.altimeter_pressure_Metric(
-                adjp, self.altitude, algorithm='aaNOAA')
-            if 'outTemp' in data:
-                data['barometer'] = weewx.wxformulas.sealevel_pressure_Metric(
-                    adjp, self.altitude, data['outTemp'])
-
-        if 'outTemp' in data:
-            if 'outHumidity' in data:
-                data['heatindex'] = weewx.wxformulas.heatindexC(
-                    data['outTemp'], data['outHumidity'])
-                data['dewpoint'] = weewx.wxformulas.dewpointC(
-                    data['outTemp'], data['outHumidity'])
-            if 'windSpeed' in data:
-                data['windchill'] = weewx.wxformulas.windchillC(
-                    data['outTemp'], data['windSpeed'])
-
-        if 'rain' in data:
-            data['rainRate'] = weewx.wxformulas.calculate_rain_rate(
-                data['rain'], data['dateTime'], self.last_rain_ts)
-            self.last_rain_ts = data['dateTime']
-
         if 'windSpeed' in data:
             if data['windSpeed'] is None or data['windSpeed'] == 0:
                 data['windDir'] = None
 
+class OWFSService(weewx.engine.StdService):
+    """Collect data from one-wire devices via owfs."""
+
+    def __init__(self, engine, config_dict):
+        """
+        interface: Where to find the one-wire sensors.  Options include
+        u, /dev/ttyS0
+        [Required. Default is u (usb)]
+
+        sensor_map: Associate sensor values with database fields.
+        [Required]
+
+        sensor_type: Indicates how data should be processed before saving.
+        [Optional. Default is gauge]
+        """
+        super(OWFSService, self).__init__(engine, config_dict)
+
+        d = config_dict.get('OWFS', {})
+        self.sensor_map = d['sensor_map']
+        self.sensor_type = d.get('sensor_type', {})
+        self.interface = d.get('interface', 'u')
+        self.max_tries = int(d.get('max_tries', 3))
+        self.retry_wait = int(d.get('retry_wait', 5))
+        self.unit_system = d.get('unit_system', 'METRIC').lower()
+        self.binding = d.get('binding', 'archive')
+        self.last_data = {}
+        self.units = weewx.US if self.unit_system == 'us' else weewx.METRIC
+
+        loginf('service version is %s' % VERSION)
+        loginf('binding is %s' % self.binding)
+        loginf('interface is %s' % self.interface)
+        loginf('sensor map is %s' % self.sensor_map)
+        loginf('sensor type map is %s' % self.sensor_type)
+        loginf('sensor unit system is %s' % self.unit_system)
+
+        ow.init(self.interface)
+        if self.binding == 'loop':
+            self.bind(weewx.NEW_LOOP_PACKET, self.handle_new_loop)
+        else:
+            self.bind(weewx.NEW_ARCHIVE_RECORD, self.handle_new_archive)
+
+    def handle_new_loop(self, event):
+        data = self.getData(event.packet)
+        event.packet.update(data)
+
+    def handle_new_archive(self, event):
+        delta = time.time() - event.record['dateTime']
+        if delta > event.record['interval'] * 60:
+            logdbg("Skipping record: time difference %s too big" % delta)
+            return
+        data = self.getData(event.record)
+        event.record.update(data)
+
+    # this implementation assumes that data from one-wire sensors are metric.
+    # if the packets to which we append are something other than metric, then
+    # we do a conversion after we have all the data.
+    def getData(self, packet):
+        ntries = 0
+        while ntries < self.max_tries:
+            ntries += 1
+            try:
+                last_data = dict(self.last_data)
+                p = {'usUnits': self.units}
+                for s in self.sensor_map:
+                    st = 'gauge'
+                    if s in self.sensor_type:
+                        st = self.sensor_type[s]
+                    if st in SENSOR_TYPES:
+                        func = SENSOR_TYPES[st]
+                        p[s] = func(s, self.sensor_map[s],
+                                    last_data, packet['dateTime'])
+                    else:
+                        logerr("unknown sensor type '%s' for %s" % (st, s))
+                self.last_data.update(last_data)
+                break
+            except ow.exError, e:
+                logerr("Failed attempt %d of %d to get onewire data: %s" %
+                       (ntries, self.max_tries, e))
+                logdbg("Waiting %d seconds before retry" % self.retry_wait)
+                time.sleep(self.retry_wait)
+        else:
+            msg = "Max retries (%d) exceeded for onewire data" % self.max_tries
+            logerr(msg)
+
+        if packet['usUnits'] != self.units:
+            converter = weewx.units.StdUnitConverters[packet['usUnits']]
+            p = converter.convertDict(p)
+        return p
+
 # define a main entry point for basic testing without weewx engine and service
 # overhead.  invoke this as follows from the weewx root dir:
 #
@@ -468,7 +579,7 @@ if __name__ == '__main__':
         (options, args) = parser.parse_args()
 
         if options.version:
-            print "owfs driver version %s" % DRIVER_VERSION
+            print "owfs version %s" % VERSION
             exit(1)
 
         # default to usb for the interface
@@ -510,7 +621,6 @@ if __name__ == '__main__':
     def traverse(device, func):
         for s in device.sensors():
             if s._type in ['DS2409']:
-                print "traversing"
                 traverse(s, func)
             else:
                 func(s)
