@@ -1,4 +1,4 @@
-# $Id: install.py 1020 2014-08-26 18:20:40Z mwall $
+# $Id: install.py 1153 2014-12-02 03:03:19Z mwall $
 # installer for OWFS
 # Copyright 2014 Matthew Wall
 
@@ -10,17 +10,15 @@ def loader():
 class OWFSInstaller(ExtensionInstaller):
     def __init__(self):
         super(OWFSInstaller, self).__init__(
-            version="0.7",
+            version="0.11",
             name='owfs',
             description='driver for one-wire devices connected via owfs',
             author="Matthew Wall",
             author_email="mwall@users.sourceforge.net",
             config={
-                'Station', {
-                    'station_type': 'OWFS'},
-                'OWFS', {
-                    'interface': 'u',
+                'OWFS': {
                     'driver': 'user.owfs',
+                    'interface': 'u',
                     'sensor_map': {},
                     'sensor_type': {}}},
             files=[('bin/user', ['bin/user/owfs.py'])]
